/*	fr.c -- FDDI Reader; write error/packet/etc. counts*/

/*	timestamp component in seconds
	timestamp component in useconds
	IP source address
	IP destination address
	IP length (16 bits) - IP protocol (8 bits) - TCP flags (8 bits)
	IP source port (16 bits)  - IP destination port (16 bits)

Special considerations:

	if plen = 65535:
		non-IP packet
		MAC level type field in dport

        if flags = 0x80
                IP fragment

        if flags = 0x40 set
                MAC in src
        if flags = 0x40 clear
                MAC in dest

Error output:

timestampsec ierrors ss_ifdrops ss_sbdrops ip_packets non-ip_pkt ip_bits
	     lossfract ave_ip_pckt_size

from if.c:
	if_ierrors	input errors on interface

from raw.c:
	ss_ifdrops	drops at/below interface
	ss_sbdrops	drops at socket buffer
*/

#include <stdio.h>
#include <errno.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <net/if.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ip_var.h>
#include <netinet/ip_icmp.h>
#include <netinet/if_fddi.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/ioctl.h>
#include <net/pfilt.h>

struct fddi_hdr {
 u_char	f_mac_bits;
 u_char	f_mac_fc;
 unsigned char	f_mac_dst[6];
 unsigned char	f_mac_src[6];
 u_char	f_llc_dsap;
 u_char	f_llc_ssap;
 u_char	f_llc_cont;
 u_char	f_llc_org[3];
 unsigned short	f_llc_etype;
};

struct fddipacket {
 struct enstamp		stamp;
 char   pad[2];
 struct fddi_hdr	fddi_h;
 struct ip ip;
 char   data[FDDIMTU];
};

struct file_format {
 struct timeval tv;
 struct  in_addr ip_src,ip_dst;
 u_short ip_len;
 u_char  ip_p;
 u_char  th_flags;
 u_short th_sport;
 u_short th_dport;
};

/* kc edited 22 mar 93 */
unsigned char d[6];   /* holds constant for comparison */

int truncation = -1;

main(argc, argv)
int argc;
char **argv;
{
 int so;
 struct ifreq ifr;
 u_int s, iphlen,pcountip=0,pcountoth=0,bcount=0;
 u_int oifdrops=0,osbdrops=0,oiferr=0;
 double lossrate;
 long otimestamp=0;
 u_short *tcpaddr;
 int cc = 60000, on = 1, len=50;
 struct fddipacket fp;
 struct timeval tv;
 FILE *fileout;
 struct file_format fmt;
 struct ctrreq ctrs;

/* SDSC E-NSS */

 d[0] = 0x08;
 d[1] = 0x00;
 d[2] = 0x5a;
 d[3] = 0x1d;
 d[4] = 0x3f;
 d[5] = 0x94;


/* UIUC E-NSS */
/*
 d[0] = 0x10;  
 d[1] = 0x00;
 d[2] = 0x5a;
 d[3] = 0xb8;
 d[4] = 0xfc;
 d[5] = 0xd2; 
*/

/* PSC E-NSS */
/*
 d[0] = 0x10;  
 d[1] = 0x00;
 d[2] = 0x5a;
 d[3] = 0xb8;
 d[4] = 0xfc;
 d[5] = 0x6e; 
*/

/* Open socket */
 so  = socket(AF_INET, SOCK_DGRAM, 0);
 if (so < 0) { perror("socket"); exit(1); }

/* Open packet filter */
 s = setup_pfilt_dev(argv[1]);
 if (s < 0) { perror(argv[1]); exit(1); }

 otimestamp = 0;
 pcountip = 1;	/* avoid divide-by-zero */

 bzero(&ctrs, sizeof ctrs);
 strcpy(ctrs.ctr_name, argv[1]);
 if (ioctl(so, SIOCRDCTRS, &ctrs) != 0) { perror("ioctl"); exit(1); }
 oiferr  =ctrs.ctr_ctrs.ctrc_fddi.fst_error;

/*
 if ((fileout = fopen("FDDI.bin", "w")) == NULL) {
  fprintf(stderr,"Unable to open file to write\n");
  exit(0);
 }
*/

 fileout = stdout;

 for (;;) {
  cc = read(s, (char *) &fp, sizeof fp);
  if(ntohs(fp.fddi_h.f_llc_etype) == 0x0800) {
   if(fp.stamp.ens_tstamp.tv_sec > otimestamp) {
    bzero(&ctrs, sizeof ctrs);
    strcpy(ctrs.ctr_name, argv[1]);
    if (ioctl(so, SIOCRDCTRS, &ctrs) != 0) { perror("ioctl"); exit(1); }
    lossrate= (1.0-((double)(pcountip+pcountoth)
              /(double)(pcountip+pcountoth+
	       ((fp.stamp.ens_ifoverflows-oifdrops)+(fp.stamp.ens_dropped)+
                (ctrs.ctr_ctrs.ctrc_fddi.fst_error-oiferr)))))*100.0;
    if ((fp.stamp.ens_tstamp.tv_sec & 0xF) == 0) {
	fprintf(stderr,
	    "time\t\terrors\tifoflow\tqdrop\tip\ttot\tbits\tlossr\tmeanpkt\n");
    }
    fprintf(stderr,"%d\t%d\t%d\t%d\t%d\t%d\t%d\t%.5f\t%.0f\n",
        fp.stamp.ens_tstamp.tv_sec,
	ctrs.ctr_ctrs.ctrc_fddi.fst_error-oiferr,
	fp.stamp.ens_ifoverflows-oifdrops,fp.stamp.ens_dropped,
        pcountip,pcountoth,bcount*8,lossrate,
	(double)((double)(bcount)/(double)(pcountip)));
    pcountip=0;pcountoth=0,bcount=0;
    oifdrops=fp.stamp.ens_ifoverflows;
    oiferr  =ctrs.ctr_ctrs.ctrc_fddi.fst_error;
    otimestamp = fp.stamp.ens_tstamp.tv_sec;
   }
   pcountip++;
   /*bcount = bcount + ntohs(fp.ip.ip_len);*/
   bcount = bcount + htons(fp.ip.ip_len);
/*
   fmt.tv = fp.stamp.ens_tstamp;
*/
   fmt.tv.tv_sec=htonl(fp.stamp.ens_tstamp.tv_sec);
   fmt.tv.tv_usec=htonl(fp.stamp.ens_tstamp.tv_usec);
   fmt.ip_src = fp.ip.ip_src;
   fmt.ip_dst = fp.ip.ip_dst;
   fmt.ip_p   = fp.ip.ip_p;
   /*fmt.ip_len = ntohs(fp.ip.ip_len);*/
   fmt.ip_len = fp.ip.ip_len;
   iphlen = fp.ip.ip_hl*4;
   /* address calculation on Alpha must be done in longs! */
   tcpaddr = (u_short *) ( ((long) &(fp.ip)) + iphlen);
   fmt.th_sport = (u_short) *tcpaddr;
   fmt.th_dport = (u_short) *(tcpaddr+1);
   fmt.th_flags = ((u_char) *(tcpaddr+6)) & 0x3f;
   if(fp.ip.ip_off > 0) {
    if(fp.ip.ip_off != 0x2000 && fp.ip.ip_off != 0x4000){fmt.th_flags=0x80;}
   }
  } else {
   pcountoth++;
/*
   fmt.tv = fp.stamp.ens_tstamp;
*/
   fmt.tv.tv_sec=htonl(fp.stamp.ens_tstamp.tv_sec);
   fmt.tv.tv_usec=htonl(fp.stamp.ens_tstamp.tv_usec);
   fmt.th_dport = fp.fddi_h.f_llc_etype;
   fmt.ip_len = 65535;
  }
  if(bcmp(fp.fddi_h.f_mac_src, d, 6) == 0){fmt.th_flags |= 0x40;}
  if((bcmp(fp.fddi_h.f_mac_src, d, 6) == 0) ||
     (bcmp(fp.fddi_h.f_mac_dst, d, 6) == 0) ) {
   if (fwrite((char *)&fmt, sizeof(fmt), 1, fileout) != 1) {
    fprintf(stderr,"Writing problem\n");
    exit(0);
   }
   fflush(fileout);
  }
 }
}

/*
 * setup_pfilt_dev - set up the packet filter
 */
int
setup_pfilt_dev(device)
char *device;
{
	int fd;
	struct timeval timeout;
	short enmode;
	short backlog = -1;	/* request the most */
	struct enfilter Filter;
	static struct ifreq ifr;	/* holds interface name	*/

	/*
	 * Open the packetfilter.  If it fails, we're out of
	 * devices.
	 */
	if ((fd = pfopen(device, 0)) < 0) {
		return(-1);
	}

	/*
	 * We want the ethernet in promiscuous mode
	 */
#ifdef	notdef
	enmode = ENBATCH|ENTSTAMP|ENNONEXCL|ENPROMISC;
#else
	enmode = ENTSTAMP|ENNONEXCL|ENPROMISC;
#endif
	if (ioctl(fd, EIOCMBIS, &enmode) < 0) {
		perror("ioctl: EIOCMBIS");
		exit(-1);
	}

#ifdef ENCOPYALL
	/*
	 * Attempt to set "copyall" mode (see our own packets).
	 * Okay if this fails.
	 */
	enmode = ENCOPYALL;
	(void) ioctl(fd, EIOCMBIS, &enmode);
#endif

	/*
	 * Set the read timeout.
	 */
	timeout.tv_sec = 1;
	timeout.tv_usec = 0;
	if (ioctl(fd, EIOCSRTIMEOUT, &timeout) < 0) {
		perror("ioctl: EIOCSRTIMEOUT");
		exit(-1);
	}

	/* set the backlog */
	if (ioctl(fd, EIOCSETW, &backlog) < 0) {
		perror("ioctl: EIOCSETW");
		exit(-1);
	}

	/* set the truncation */
	if (truncation >= 0) {
	    if (ioctl(fd, EIOCTRUNCATE, &truncation) < 0) {
		perror("ioctl: EIOCTRUNCATE");
		exit(-1);
	    }
	}

	/* accept all packets */
	Filter.enf_Priority = 37;	/* anything > 2 */
	Filter.enf_FilterLen = 0;	/* means "always true" */
	if (ioctl(fd, EIOCSETF, &Filter) < 0) {
		perror("ioctl: EIOCSETF");
		exit(-1);
	}

	return(fd);
}
