/* 
#
# Convert fr output to ASCII
#
#    sec
#    usec
#    src
#    dst
#    prot - empty - length(2)
#    sport
#    dport
#
#        timestamp component in seconds
#        timestamp component in useconds
#        IP source address
#        IP destination address
#        IP length (16 bits) - IP protocol (8 bits) - TCP flags (8 bits)
#        IP source port (16 bits)  - IP destination port (16 bits)
#
# Special considerations:
#
#        if plen = 65535:
#                non-IP packet
#                MAC level type field in dport
#
#        if flags = 0xff
#                IP fragment
*/

#include <fcntl.h>
#include <stdio.h>

main(argc, argv)
int argc;
char *argv[];
{
 int fd;
 u_int c,i,flags,prot,miscf,ports,sport,dport,dsth,srch,plen;
 u_int s1,s2,s3,s4,d1,d2,d3,d4;
 u_int record[6],tstampsec,tstampusec;
 double time,orgtime,oldtime;
 char fname[80];
 extern char *optarg;

 while ((c = getopt(argc, argv, "f:")) != -1) {
  switch (c) {
  case 'f':
   (void)strcpy(fname,optarg);
   break;
  }
 }

 if (argc >= 2) {
  fd = open(fname, O_RDONLY);
  if (fd < 0) {
   perror("Unable to open input file");
   exit(1);
  }
 } else {
  fd = 0;
   if (fd < 0) {
   perror("Unable to open STDIN");
   exit(1);
  }
 }

 if (read(fd, (char *) record, sizeof(record)) == 0) {
  perror("Unable to read input file");
  exit(1);
 }
 tstampsec  = ntohl(record[0]);
 tstampusec = ntohl(record[1]);
 orgtime=(double)tstampsec+(((double)tstampusec)/1000000);
 oldtime=orgtime;
 
 /* Start reading input file packet entries */
 while (1) {
next:
  if (read(fd, (char *) record, sizeof(record)) == 0) {
   exit(1);
  }

  miscf=ntohl(record[4]);
  plen = miscf / 65536;

  if (plen == 65535) {goto next;}

  tstampsec  = ntohl(record[0]);
  tstampusec = ntohl(record[1]);
  time=(double)tstampsec+(((double)tstampusec)/1000000);

  srch =ntohl(record[2]);
  s1=(srch>>24) & 255;
  s2=(srch>>16) & 255;
  s3=(srch>>8) & 255;
  s4=srch & 255;

  dsth =ntohl(record[3]);
  d1=(dsth>>24) & 255;
  d2=(dsth>>16) & 255;
  d3=(dsth>>8) & 255;
  d4=dsth & 255;

  flags=(miscf & 0x000000ff);
  prot =(miscf & 0x0000ff00)>>8;

  ports=ntohl(record[5]);
  sport=(ports & 0xffff0000)>>16;
  dport= ports & 0x0000ffff;

  printf("%.6f\t%d\t%d\t%d.%d.%d.%d-%d.%d.%d.%d\t%d\t%d\t%d\t%x\t",
   time,(int)(((time-oldtime)*1000000.)+0.5),plen,
   s1,s2,s3,s4,d1,d2,d3,d4,prot,sport,dport,flags&0x3f);
  if((flags&0x40) == 0x40) {printf("S\t");}else{printf("D\t");}
  if((flags&0x80) == 0x80) {printf("F\n");}else{printf(".\n");}
  oldtime=time;
 }
}
